'use client';

import * as React from 'react';
import { CheckIcon, ChevronsUpDownIcon } from 'lucide-react';
import { cn } from '@/lib/utils';
import { Button, type buttonVariants } from '@/components/ui/button';
import { type VariantProps } from 'class-variance-authority';
import {
  Command,
  CommandEmpty,
  CommandGroup,
  CommandInput,
  CommandItem,
  CommandList,
} from '@/components/ui/command';
import {
  Popover,
  PopoverContent,
  PopoverTrigger,
} from '@/components/ui/popover';

type ComboboxProps = {
  selectOptions: {
    value: string;
    label: string;
  }[];
  optionName?: string;
  buttonProps?: React.ComponentProps<typeof Button> &
    VariantProps<typeof buttonVariants>;
  chevronProps?: React.ComponentProps<typeof ChevronsUpDownIcon>;
  popoverContentProps?: React.ComponentProps<typeof PopoverContent>;
  checkIconClassName?: string;
};

export const Combobox = ({
  selectOptions,
  optionName = 'options',
  buttonProps = {
    className: 'w-[200px] justify-between',
    variant: 'outline',
  },
  chevronProps = {
    className: 'ml-2 h-4 w-4 shrink-0 opacity-50',
  },
  popoverContentProps = {
    className: 'w-[200px] p-0',
  },
  checkIconClassName = 'mr-2 h-4 w-4',
}: ComboboxProps) => {
  const [open, setOpen] = React.useState(false);
  const [value, setValue] = React.useState('');

  return (
    <Popover open={open} onOpenChange={setOpen}>
      <PopoverTrigger asChild>
        <Button role='combobox' aria-expanded={open} {...buttonProps}>
          {value
            ? selectOptions.find((selectOption) => selectOption.value === value)
                ?.label
            : `Select ${optionName}`}
          <ChevronsUpDownIcon {...chevronProps} />
        </Button>
      </PopoverTrigger>
      <PopoverContent {...popoverContentProps}>
        <Command>
          <CommandInput placeholder={`Search ${optionName}`} />
          <CommandList>
            <CommandEmpty>{`No ${optionName} found.`}</CommandEmpty>
            <CommandGroup>
              {selectOptions.map((selectOption) => (
                <CommandItem
                  key={selectOption.value}
                  value={selectOption.value}
                  onSelect={(currentValue) => {
                    setValue(currentValue === value ? '' : currentValue);
                    setOpen(false);
                  }}
                >
                  <CheckIcon
                    className={cn(
                      checkIconClassName,
                      value === selectOption.value
                        ? 'opacity-100'
                        : 'opacity-0',
                    )}
                  />
                  {selectOption.label}
                </CommandItem>
              ))}
            </CommandGroup>
          </CommandList>
        </Command>
      </PopoverContent>
    </Popover>
  );
};
